#!/bin/bash

echo ====================================
echo "        Recipes and Tests"
echo "           Linux/MacOS"
echo ------------------------------------
echo

if ! [ -d Tests ] && [ -d "Capitoline Hashes" ] && [ -d ROMs ] && [ -d Components ] && [ -f capcli.Linux ]
then
   echo ERROR: Critical direcories appear to be missing
   echo NOTE: Please run ./Tests/rat.bash in a console from the main Capitoline directory
   exit
fi

if [ `uname` = 'Darwin' ]
then
   CAPCLI=./capcli.MacOS
   CRC32=./crc32.MacOS
else
   CAPCLI=./capcli.Linux
   CRC32=./crc32.Linux
fi

OK=`printf '\033[0;32mOK\033[0m'`
FAILED=`printf '\033[0;31mFAILED\033[0m'`

TESTCNT=0
TESTOK=0

rm Tests/Logs/Files.expected 2>/dev/null
rm Tests/Logs/Files.OK 2>/dev/null
rm Tests/Logs/Files.Failed 2>/dev/null

for TEST in `grep -v ^# Tests/Tests.txt`
do
   echo ====================================
   echo "$TEST"
   echo ------------------------------------
   echo Deleting any previous output...
   cat Tests/$TEST.expected|while read ECRC FILE
   do
      rm "$FILE" 2>/dev/null
   done
   echo Running test...
   TESTCNT=$(( $TESTCNT + 1 ))
   cat Tests/$TEST.test | $CAPCLI >Tests/Logs/$TEST.log 2>&1
   echo Checking output files...
   rm Tests/Logs/Test.Failed 2>/dev/null
   cat Tests/$TEST.expected|while read ECRC FILE
   do
      echo -n `basename "$FILE"`... 
      echo "$FILE" >>Tests/Logs/Files.expected
      if [ -f "$FILE" ]
      then
         ACRC=`$CRC32 "$FILE"`
         if [ "$ACRC" = "$ECRC" ]
         then
            echo $OK
            echo "$FILE" >>Tests/Logs/Files.OK
         else
            echo $FAILED: CRC mismatch
            echo "$ACRC != $ECRC $FILE" >>Tests/Logs/Files.Failed
            touch Tests/Logs/Test.Failed
         fi
      else
         echo $FAILED: no file created
         echo "Missing $FILE" >>Tests/Logs/Files.Failed
         touch Tests/Logs/Test.Failed
      fi
   done
   if [ -f Tests/Logs/Test.Failed ]
   then
      echo Test $FAILED
   else
      echo Test $OK
      TESTOK=$(( $TESTOK + 1 ))
   fi
done

   echo ====================================
   echo Summary
   echo ------------------------------------
FOK=`cat Tests/Logs/Files.OK|wc -l`
FEX=`cat Tests/Logs/Files.expected|wc -l`
if [ "$FOK" = "$FEX" ]
then
   echo $FOK files of $FEX $OK
else
   echo $(( $FEX - $FOK )) of $FEX $FAILED
fi

if [ "$TESTOK" = "$TESTCNT" ]
then
   echo $TESTOK tests of $TESTCNT $OK
else
   echo $(( $TESTCNT - $TESTOK )) of $TESTCNT $FAILED
fi
